package cc.toph.simplycompat.item;

import cc.toph.simplycompat.util.ToolMaterials;
import cc.toph.simplycompat.util.WeaponType;
import net.sweenus.simplyswords.item.SimplySwordsSwordItem;

/**
 * Extended Class from Simply Swords to store sword type.
 * Storing Sword type will let us avoid all the static mess in registries.
 * This class stores all the data we need from the sword, so we can just
 * retrieve the object from the registries cast it and do whatever we need.
 */
public class SimplyCompatSwordItem<E extends Enum<E> & ToolMaterials<E>> extends SimplySwordsSwordItem {

    /**
     * The type of the weapon.
     */
    private final WeaponType TYPE;
    private final ToolMaterials<E> MATERIAL;

    /**
     * Constructor for SimplyCompatSwordItem.
     *
     * @param toolMaterial The material of the tool, which some of its properties/stats.
     * @param type         The type of the weapon, which affects its modifiers.
     * @param <E>          The type of the enum that extends ToolMaterials.
     */
    public SimplyCompatSwordItem(ToolMaterials<E> material, WeaponType type) {
        super(
                material,
                (int) (material.getDamageModifier() + type.getPositiveModifier() - type.getNegativeModifier()),
                type.getAttackSpeed(),
                material.getRepairIngredient().toString()
        );

        this.TYPE = type;
        this.MATERIAL = material;
    }

    public ToolMaterials<E> getMaterial() {
        return MATERIAL;
    }

    public WeaponType getType() {
        return TYPE;
    }
}