package cc.toph.simplycompat.registry;

import cc.toph.simplycompat.SimplyCompat;
import cc.toph.simplycompat.item.SimplyCompatSwordItem;
import cc.toph.simplycompat.item.SimplyCompatToolMaterials;
import cc.toph.simplycompat.util.ToolMaterials;
import cc.toph.simplycompat.util.WeaponType;
import dev.architectury.registry.registries.DeferredRegister;
import net.minecraft.core.registries.Registries;
import net.minecraft.world.item.Item;

public final class ItemsRegistry {

    public static final DeferredRegister<Item> SWORDS = DeferredRegister.create(SimplyCompat.MOD_ID, Registries.ITEM);

    /**
     * Registers a sword item with the given material and weapon type.
     *
     * @param material The material of the tool, which some of its properties/stats.
     * @param type     The type of the weapon, which affects its modifiers.
     * @param <E>      The type of the enum that extends ToolMaterials.
     */
    public static <E extends Enum<E> & ToolMaterials<E>> void registerSword(
            ToolMaterials<E> material,
            WeaponType type
    ) {
        String name = material.getName() + "_" + type.getWeaponNameSuffix();
        SWORDS.register(name, () ->
                new SimplyCompatSwordItem<>(material, type));
    }

    /**
     * Registers all sword items. Iterate through all available weapon types
     * and tool materials. For each combination, the corresponding sword
     * is registered using the private {@code registerSword} method.
     * <br>
     * This ensures that each defined material and weapon type combination is
     * registered within the item registry for use within the mod.
     */
    public static void registerSwords() {

        // Register "Vanilla" swords
        for (WeaponType type : WeaponType.values()) {
            for (SimplyCompatToolMaterials material : SimplyCompatToolMaterials.values()) {
                if (material.isEnabled()) registerSword(material, type);
            }
        }

        CompatRegistry.registerAll();
        SWORDS.register();
    }
}
