package cc.toph.simplycompat.forge.datagen;

import cc.toph.simplycompat.SimplyCompat;
import cc.toph.simplycompat.item.SimplyCompatSwordItem;
import cc.toph.simplycompat.registry.ItemsRegistry;
import net.minecraft.advancements.critereon.InventoryChangeTrigger;
import net.minecraft.core.registries.BuiltInRegistries;
import net.minecraft.data.PackOutput;
import net.minecraft.data.recipes.FinishedRecipe;
import net.minecraft.data.recipes.RecipeCategory;
import net.minecraft.data.recipes.RecipeProvider;
import net.minecraft.data.recipes.ShapedRecipeBuilder;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.Items;
import net.minecraftforge.common.Tags;
import org.jetbrains.annotations.NotNull;

import java.util.function.Consumer;

public class SimplyCompatRecipes extends RecipeProvider {
    public SimplyCompatRecipes(PackOutput packOutput) {
        super(packOutput);
    }

    @Override
    protected void buildRecipes(@NotNull Consumer<FinishedRecipe> consumer) {
        ItemsRegistry.SWORDS.forEach(sword -> buildRecipe(BuiltInRegistries.ITEM.get(sword.getId()), consumer));
    }

    private void buildRecipe(Item sword, Consumer<FinishedRecipe> consumer) {

        if (sword instanceof SimplyCompatSwordItem<?> SCSword) {

            // TODO: Some swords wont follow the usual recipe
            // if (sword.getId() != ) {}

            var material = SCSword.getMaterial();
            switch (SCSword.getType()) {
                case CHAKRAM -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
                        .pattern("YXY")
                        .pattern("X X")
                        .pattern("Y#Y")
                        .define('Y', Items.IRON_NUGGET)
                        .define('X', SCSword.getTier().getRepairIngredient())
                        .define('#', Tags.Items.RODS_WOODEN)
                        .group(SimplyCompat.MOD_ID)
                        .unlockedBy("has_" + material.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
                                BuiltInRegistries.ITEM.get(material.getIngredientLocation())))
                        //      ItemPredicate.Builder.item().of(Tags.Items.GEMS_DIAMOND).build()))
                        .save(consumer);
                case CLAYMORE -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
                        .pattern(" YX")
                        .pattern("YXY")
                        .pattern("#Y ")
                        .define('Y', Items.IRON_NUGGET)
                        .define('X', SCSword.getTier().getRepairIngredient())
                        .define('#', Tags.Items.RODS_WOODEN)
                        .group(SimplyCompat.MOD_ID)
                        .unlockedBy("has_" + material.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
                                BuiltInRegistries.ITEM.get(material.getIngredientLocation())))
                        .save(consumer);
                case CUTLASS -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
                        .pattern(" Y")
                        .pattern("XX")
                        .pattern("# ")
                        .define('Y', Items.IRON_NUGGET)
                        .define('X', SCSword.getTier().getRepairIngredient())
                        .define('#', Tags.Items.RODS_WOODEN)
                        .group(SimplyCompat.MOD_ID)
                        .unlockedBy("has_" + material.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
                                BuiltInRegistries.ITEM.get(material.getIngredientLocation())))
                        .save(consumer);
                case GLAIVE -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
                        .pattern("  X")
                        .pattern(" #X")
                        .pattern("#  ")
                        .define('X', SCSword.getTier().getRepairIngredient())
                        .define('#', Tags.Items.RODS_WOODEN)
                        .group(SimplyCompat.MOD_ID)
                        .unlockedBy("has_" + material.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
                                BuiltInRegistries.ITEM.get(material.getIngredientLocation())))
                        .save(consumer);
                case GREATAXE -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
                        .pattern("XXX")
                        .pattern("Y#Y")
                        .pattern(" # ")
                        .define('Y', Items.IRON_NUGGET)
                        .define('X', SCSword.getTier().getRepairIngredient())
                        .define('#', Tags.Items.RODS_WOODEN)
                        .group(SimplyCompat.MOD_ID)
                        .unlockedBy("has_" + material.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
                                BuiltInRegistries.ITEM.get(material.getIngredientLocation())))
                        .save(consumer);
                case GREATHAMMER -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
                        .pattern("XXX")
                        .pattern("YYY")
                        .pattern(" # ")
                        .define('Y', Items.IRON_NUGGET)
                        .define('X', SCSword.getTier().getRepairIngredient())
                        .define('#', Tags.Items.RODS_WOODEN)
                        .group(SimplyCompat.MOD_ID)
                        .unlockedBy("has_" + material.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
                                BuiltInRegistries.ITEM.get(material.getIngredientLocation())))
                        .save(consumer);
                case HALBERD -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
                        .pattern(" XY")
                        .pattern("X#X")
                        .pattern("#  ")
                        .define('Y', Items.IRON_NUGGET)
                        .define('X', SCSword.getTier().getRepairIngredient())
                        .define('#', Tags.Items.RODS_WOODEN)
                        .group(SimplyCompat.MOD_ID)
                        .unlockedBy("has_" + material.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
                                BuiltInRegistries.ITEM.get(material.getIngredientLocation())))
                        .save(consumer);
                case KATANA -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
                        .pattern("   ")
                        .pattern("#XX")
                        .pattern("   ")
                        .define('X', SCSword.getTier().getRepairIngredient())
                        .define('#', Tags.Items.RODS_WOODEN)
                        .group(SimplyCompat.MOD_ID)
                        .unlockedBy("has_" + material.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
                                BuiltInRegistries.ITEM.get(material.getIngredientLocation())))
                        .save(consumer);
                case LONGSWORD -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
                        .pattern("#  ")
                        .pattern(" X ")
                        .pattern("  X")
                        .define('X', SCSword.getTier().getRepairIngredient())
                        .define('#', Tags.Items.RODS_WOODEN)
                        .group(SimplyCompat.MOD_ID)
                        .unlockedBy("has_" + material.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
                                BuiltInRegistries.ITEM.get(material.getIngredientLocation())))
                        .save(consumer);
                case RAPIER -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
                        .pattern("  X")
                        .pattern(" X ")
                        .pattern("#  ")
                        .define('X', SCSword.getTier().getRepairIngredient())
                        .define('#', Tags.Items.RODS_WOODEN)
                        .group(SimplyCompat.MOD_ID)
                        .unlockedBy("has_" + material.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
                                BuiltInRegistries.ITEM.get(material.getIngredientLocation())))
                        .save(consumer);
                case SAI -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
                        .pattern("  ")
                        .pattern(" X")
                        .pattern("# ")
                        .define('X', SCSword.getTier().getRepairIngredient())
                        .define('#', Tags.Items.RODS_WOODEN)
                        .group(SimplyCompat.MOD_ID)
                        .unlockedBy("has_" + material.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
                                BuiltInRegistries.ITEM.get(material.getIngredientLocation())))
                        .save(consumer);
                case SCYTHE -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
                        .pattern("XX#")
                        .pattern("X# ")
                        .pattern("#  ")
                        .define('X', SCSword.getTier().getRepairIngredient())
                        .define('#', Tags.Items.RODS_WOODEN)
                        .group(SimplyCompat.MOD_ID)
                        .unlockedBy("has_" + material.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
                                BuiltInRegistries.ITEM.get(material.getIngredientLocation())))
                        .save(consumer);
                case SPEAR -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
                        .pattern("  X")
                        .pattern(" # ")
                        .pattern("#  ")
                        .define('X', SCSword.getTier().getRepairIngredient())
                        .define('#', Tags.Items.RODS_WOODEN)
                        .group(SimplyCompat.MOD_ID)
                        .unlockedBy("has_" + material.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
                                BuiltInRegistries.ITEM.get(material.getIngredientLocation())))
                        .save(consumer);
                case TWINBLADE -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
                        .pattern("  X")
                        .pattern(" # ")
                        .pattern("X  ")
                        .define('X', SCSword.getTier().getRepairIngredient())
                        .define('#', Tags.Items.RODS_WOODEN)
                        .group(SimplyCompat.MOD_ID)
                        .unlockedBy("has_" + material.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
                                BuiltInRegistries.ITEM.get(material.getIngredientLocation())))
                        .save(consumer);
                case WARGLAIVE -> ShapedRecipeBuilder.shaped(RecipeCategory.TOOLS, SCSword)
                        .pattern(" Y ")
                        .pattern("X#X")
                        .pattern("   ")
                        .define('Y', Items.IRON_NUGGET)
                        .define('X', SCSword.getTier().getRepairIngredient())
                        .define('#', Tags.Items.RODS_WOODEN)
                        .group(SimplyCompat.MOD_ID)
                        .unlockedBy("has_" + material.getName().toLowerCase(), InventoryChangeTrigger.TriggerInstance.hasItems(
                                BuiltInRegistries.ITEM.get(material.getIngredientLocation())))
                        .save(consumer);
                default -> throw new IllegalStateException("Unexpected value: " + SCSword.getType());
            }
        } else {
            throw new IllegalStateException("Unexpected item type: " + sword.getClass().getName() + "or");
        }
    }
}